#ifndef MSP_GLTK_GEOMETRY_H_
#define MSP_GLTK_GEOMETRY_H_

#include <msp/datafile/objectloader.h>

namespace Msp {
namespace GLtk {

/**
Specifies the position and size of a widget or graphic.
*/
struct Geometry
{
	int x, y;
	unsigned w, h;

	Geometry(): x(0), y(0), w(1), h(1) { }
	Geometry(int x_, int y_, unsigned w_, unsigned h_): x(x_), y(y_), w(w_), h(h_) { }
	bool is_inside(int, int) const;
	bool is_inside_relative(int, int) const;
};


/**
Specifies margins on the sides of an element.
*/
struct Sides
{
	class Loader: public DataFile::ObjectLoader<Sides>
	{
	public:
		Loader(Sides &);
	private:
		void horizontal(unsigned);
		void vertical(unsigned);
	};

	unsigned top;
	unsigned right;
	unsigned bottom;
	unsigned left;

	Sides();
	Sides(unsigned);
	Sides(unsigned, unsigned);
	Sides(unsigned, unsigned, unsigned);
	Sides(unsigned, unsigned, unsigned, unsigned);
};


/**
Performs alignment of nested geometries, such as widgets and their parts.
*/
struct Alignment
{
	float x, y;
	float w, h;

	Alignment(): x(0), y(0), w(1), h(1) { }
	void apply(Geometry &, const Geometry &) const;
	void apply(Geometry &, const Geometry &, const Sides &) const;
};

} // namespace GLtk
} // namespace Msp

#endif
