#ifndef MSP_GLTK_PANEL_H_
#define MSP_GLTK_PANEL_H_

#include "container.h"
#include "layout.h"

namespace Msp {
namespace GLtk {

/**
Panels are containers for other widgets.  Panel styles should have a special
part "children" to render the child widgets.  All properties of this part are
ignored.
*/
class Panel: public Container
{
public:
	class Loader: public DataFile::DerivedObjectLoader<Panel, Widget::Loader>
	{
	public:
		typedef std::map<std::string, Widget *> WidgetMap;

	protected:
		WidgetMap &wdg_map;
		Widget *last_widget;
	
	public:
		Loader(Panel &, WidgetMap &);
	private:
		Layout &get_layout();
		Widget &get_last_widget();
		template<typename T>
		void arrangement();
		template<typename T>
		void child(const std::string &);
		void constraint(Layout::ConstraintType, const std::string &);
		void expand(bool, bool);
		void ghost(bool);
		void gravity(int, int);
		void grid(unsigned);
		void layout();
		void panel(const std::string &);
	};

private:
	template<typename T>
	class ArrangedLoader: public DataFile::Loader
	{
	private:
		typename T::Loader arr_loader;

	public:
		ArrangedLoader(Loader &, T &);
	};

protected:
	Layout *layout;

	Panel(const Panel &);
	Panel &operator=(const Panel &);
public:
	Panel();
	virtual ~Panel();

	virtual const char *get_class() const { return "panel"; }

	void set_layout(Layout *);
	Layout *get_layout() { return layout; }

protected:
	virtual void autosize_special(const Part &, Geometry &);
	virtual void render_special(const Part &, GL::Renderer &) const;

	virtual void on_geometry_change();
	virtual void on_child_added(Widget &);
	virtual void on_child_removed(Widget &);
};

} // namespace GLtk
} // namespace Msp

#endif
