#ifndef MSP_GLTK_TOGGLE_H_
#define MSP_GLTK_TOGGLE_H_

#include <sigc++/signal.h>
#include "text.h"
#include "widget.h"

namespace Msp {
namespace GLtk {

/**
Allows toggling a value between two states.
*/
class Toggle: public Widget
{
public:
	class Loader: public DataFile::DerivedObjectLoader<Toggle, Widget::Loader>
	{
	public:
		Loader(Toggle &);
	private:
		virtual void finish();
		void text(const std::string &);
	};

	sigc::signal<void, bool> signal_toggled;

private:
	Text text;
	bool pressed;
	bool value;
	bool exclusive;

public:
	Toggle(const std::string & = std::string());

	virtual const char *get_class() const { return "toggle"; }

private:
	virtual void autosize_special(const Part &, Geometry &);

public:
	void set_text(const std::string &);
	void set_exclusive(bool);
	bool get_exclusive() const { return exclusive; }
private:
	void exclude_siblings();
public:
	void set_value(bool);
	bool get_value() const { return value; }

private:
	virtual void rebuild_special(const Part &);

public:
	virtual void button_press(int, int, unsigned);
	virtual void button_release(int, int, unsigned);
private:
	virtual void on_style_change();
};

} // namespace GLtk
} // namespace Msp

#endif
